@@ -2,8 +2,12 @@ from __future__ import annotations
 
 import ast
 import asyncio
+import itertools
 import os
+import shutil
+import sys
 import weakref
+import zlib
 from abc import ABC, abstractmethod
 from collections import OrderedDict
 from concurrent.futures import ProcessPoolExecutor
@@ -23,8 +27,11 @@ from typing import (
     final,
 )
 
+from ....__version__ import __version__
 from ....utils.async_cache import AsyncSimpleLRUCache
 from ....utils.async_tools import Lock, async_tasking_event, create_sub_task, threaded
+from ....utils.dataclasses import as_json, from_json
+from ....utils.glob_path import iter_files
 from ....utils.logging import LoggingDescriptor
 from ....utils.path import path_is_relative_to
 from ....utils.uri import Uri
@@ -32,18 +39,17 @@ from ...common.decorators import language_id
 from ...common.lsp_types import DocumentUri, FileChangeType, FileEvent
 from ...common.parts.workspace import FileWatcherEntry, Workspace
 from ...common.text_document import TextDocument
-from ..configuration import RobotConfig
+from ..configuration import CacheSaveLocation, RobotCodeConfig
 from ..utils.ast_utils import HasError, HasErrors, Token
 from ..utils.async_ast import walk
 from ..utils.robot_path import find_file_ex
-from ..utils.version import get_robot_version
+from ..utils.version import get_robot_version, get_robot_version_str
 from .entities import CommandLineVariableDefinition, VariableDefinition
 
 if TYPE_CHECKING:
     from ..protocol import RobotLanguageServerProtocol
     from .namespace import Namespace
 
-
 from .library_doc import (
     ROBOT_LIBRARY_PACKAGE,
     ArgumentSpec,
@@ -53,6 +59,8 @@ from .library_doc import (
     KeywordDoc,
     KeywordStore,
     LibraryDoc,
+    LibraryType,
+    ModuleSpec,
     VariablesDoc,
     complete_library_import,
     complete_resource_import,
@@ -61,6 +69,7 @@ from .library_doc import (
     find_library,
     find_variables,
     get_library_doc,
+    get_module_spec,
     get_variables_doc,
     is_embedded_keyword,
     is_library_by_path,
@@ -451,14 +460,59 @@ class _VariablesEntry(_ImportEntry):
             return self._lib_doc
 
 
+@dataclass
+class LibraryMetaData:
+    meta_version: str
+    name: Optional[str]
+    origin: Optional[str]
+    submodule_search_locations: Optional[List[str]]
+    by_path: bool
+
+    mtimes: Optional[Dict[str, int]] = None
+
+    @property
+    def filepath_base(self) -> Path:
+        if self.by_path:
+            if self.origin is not None:
+                p = Path(self.origin)
+
+                return Path(f"{zlib.adler32(str(p.parent).encode('utf-8')):08x}_{p.stem}")
+        else:
+            if self.name is not None:
+                return Path(self.name.replace(".", "/"))
+
+        raise ValueError("Cannot determine filepath base.")
+
+
 class ImportsManager:
     _logger = LoggingDescriptor()
 
-    def __init__(self, parent_protocol: RobotLanguageServerProtocol, folder: Uri, config: RobotConfig) -> None:
+    def __init__(self, parent_protocol: RobotLanguageServerProtocol, folder: Uri, config: RobotCodeConfig) -> None:
         super().__init__()
         self.parent_protocol = parent_protocol
+
         self.folder = folder
-        self.config: RobotConfig = config
+        get_robot_version()
+
+        cache_base_path = self.folder.to_path()
+        if (
+            config.analysis.cache.save_location == CacheSaveLocation.WORKSPACE_STORAGE
+            and isinstance(self.parent_protocol.initialization_options, dict)
+            and "storageUri" in self.parent_protocol.initialization_options
+        ):
+            cache_base_path = Uri(self.parent_protocol.initialization_options["storageUri"]).to_path()
+
+        self._logger.trace(lambda: f"use {cache_base_path} as base for caching")
+
+        self.cache_path = cache_base_path / ".robotcode_cache"
+
+        self.lib_doc_cache_path = (
+            self.cache_path
+            / f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"
+            / get_robot_version_str()
+            / "libdoc"
+        )
+        self.config = config
         self._libaries_lock = Lock()
         self._libaries: OrderedDict[_LibrariesEntryKey, _LibrariesEntry] = OrderedDict()
         self._resources_lock = Lock()
@@ -472,7 +526,7 @@ class ImportsManager:
         self._command_line_variables_lock = Lock()
 
         self._environment = dict(os.environ)
-        self._environment.update(self.config.env)
+        self._environment.update(self.config.robot.env)
 
         self._library_files_cache = AsyncSimpleLRUCache()
         self._resource_files_cache = AsyncSimpleLRUCache()
@@ -482,6 +536,11 @@ class ImportsManager:
     def environment(self) -> Mapping[str, str]:
         return self._environment
 
+    def clear_cache(self) -> None:
+        if self.cache_path.exists():
+            shutil.rmtree(self.cache_path)
+            self._logger.debug(f"Cleared cache {self.cache_path}")
+
     @_logger.call
     async def get_command_line_variables(self) -> List[VariableDefinition]:
         from robot.utils.text import split_args_from_name_or_path
@@ -491,14 +550,14 @@ class ImportsManager:
             if self._command_line_variables is None:
                 command_line_vars: List[VariableDefinition] = []
 
-                if self.config is None:
+                if self.config.robot is None:
                     self._command_line_variables = []
                 else:
                     command_line_vars = [
                         CommandLineVariableDefinition(0, 0, 0, 0, "", f"${{{k}}}", None, has_value=True, value=(v,))
-                        for k, v in self.config.variables.items()
+                        for k, v in self.config.robot.variables.items()
                     ]
-                    for variable_file in self.config.variable_files:
+                    for variable_file in self.config.robot.variable_files:
                         name, args = split_args_from_name_or_path(variable_file)
                         try:
                             lib_doc = await self.get_libdoc_for_variables_import(
@@ -693,6 +752,58 @@ class ImportsManager:
         except RuntimeError:
             pass
 
+    async def get_library_meta(
+        self,
+        name: str,
+        base_dir: str = ".",
+        variables: Optional[Dict[str, Optional[Any]]] = None,
+    ) -> Tuple[Optional[LibraryMetaData], str]:
+        try:
+            import_name = await self.find_library(
+                name,
+                base_dir=base_dir,
+                variables=variables,
+            )
+
+            result: Optional[LibraryMetaData] = None
+            module_spec: Optional[ModuleSpec] = None
+            if is_library_by_path(import_name):
+                if (p := Path(import_name)).exists():
+                    result = LibraryMetaData(__version__, p.stem, import_name, None, True)
+            else:
+                module_spec = get_module_spec(import_name)
+                if module_spec is not None and module_spec.origin is not None:
+                    result = LibraryMetaData(
+                        __version__,
+                        module_spec.name,
+                        module_spec.origin,
+                        module_spec.submodule_search_locations,
+                        False,
+                    )
+            if result is not None:
+                if result.origin is not None:
+                    result.mtimes = {result.origin: Path(result.origin).resolve().stat().st_mtime_ns}
+
+                if result.submodule_search_locations:
+                    if result.mtimes is None:
+                        result.mtimes = {}
+                    result.mtimes.update(
+                        {
+                            str(f): f.resolve().stat().st_mtime_ns
+                            for f in itertools.chain(
+                                *(iter_files(loc, "**/*.py") for loc in result.submodule_search_locations)
+                            )
+                        }
+                    )
+
+            return result, import_name
+        except (SystemExit, KeyboardInterrupt):
+            raise
+        except BaseException:
+            pass
+
+        return None, import_name
+
     async def find_library(self, name: str, base_dir: str, variables: Optional[Dict[str, Any]] = None) -> str:
         return await self._library_files_cache.get(self._find_library, name, base_dir, variables)
 
@@ -705,7 +816,7 @@ class ImportsManager:
                 name,
                 str(self.folder.to_path()),
                 base_dir,
-                self.config.variables if self.config is not None else None,
+                self.config.robot.variables if self.config.robot is not None else None,
                 variables,
             )
 
@@ -735,7 +846,7 @@ class ImportsManager:
                 name,
                 str(self.folder.to_path()),
                 base_dir,
-                self.config.variables if self.config is not None else None,
+                self.config.robot.variables if self.config.robot is not None else None,
                 variables,
                 file_type,
             )
@@ -754,7 +865,7 @@ class ImportsManager:
                 name,
                 str(self.folder.to_path()),
                 base_dir,
-                self.config.variables if self.config is not None else None,
+                self.config.robot.variables if self.config.robot is not None else None,
                 variables,
             )
 
@@ -783,7 +894,30 @@ class ImportsManager:
         )
 
         async def _get_libdoc() -> LibraryDoc:
+
+            meta, source = await self.get_library_meta(
+                name,
+                base_dir,
+                variables,
+            )
+
             self._logger.debug(lambda: f"Load Library {source}{repr(args)}")
+            if meta is not None:
+                meta_file = Path(self.lib_doc_cache_path, meta.filepath_base.with_suffix(".meta.json"))
+                if meta_file.exists():
+                    try:
+                        saved_meta = from_json(meta_file.read_text("utf-8"), LibraryMetaData)
+                        if saved_meta == meta:
+                            return from_json(
+                                Path(self.lib_doc_cache_path, meta.filepath_base.with_suffix(".spec.json")).read_text(
+                                    "utf-8"
+                                ),
+                                LibraryDoc,
+                            )
+                    except (SystemExit, KeyboardInterrupt):
+                        raise
+                    except BaseException as e:
+                        self._logger.exception(e)
 
             with ProcessPoolExecutor(max_workers=1) as executor:
                 result = await asyncio.wait_for(
@@ -794,7 +928,7 @@ class ImportsManager:
                         args,
                         str(self.folder.to_path()),
                         base_dir,
-                        self.config.variables if self.config is not None else None,
+                        self.config.robot.variables if self.config.robot is not None else None,
                         variables,
                     ),
                     LOAD_LIBRARY_TIME_OUT,
@@ -804,6 +938,20 @@ class ImportsManager:
                     self._logger.warning(
                         lambda: f"stdout captured at loading library {name}{repr(args)}:\n{result.stdout}"
                     )
+                try:
+                    if meta is not None and result.library_type in [LibraryType.CLASS, LibraryType.MODULE]:
+                        meta_file = Path(self.lib_doc_cache_path, meta.filepath_base.with_suffix(".meta.json"))
+                        meta_file.parent.mkdir(parents=True, exist_ok=True)
+                        meta_file.write_text(as_json(meta), "utf-8")
+
+                        spec_file = Path(self.lib_doc_cache_path, meta.filepath_base.with_suffix(".spec.json"))
+                        spec_file.write_text(as_json(result), "utf-8")
+                    else:
+                        self._logger.debug(lambda: f"Skip caching library {name}{repr(args)}")
+                except (SystemExit, KeyboardInterrupt):
+                    raise
+                except BaseException as e:
+                    self._logger.exception(e)
 
                 return result
 
@@ -921,9 +1069,9 @@ class ImportsManager:
             keywords=[
                 KeywordDoc(
                     name=kw[0].name,
-                    args=tuple(KeywordArgumentDoc.from_robot(a) for a in kw[0].args),
+                    args=list(KeywordArgumentDoc.from_robot(a) for a in kw[0].args),
                     doc=kw[0].doc,
-                    tags=tuple(kw[0].tags),
+                    tags=list(kw[0].tags),
                     source=kw[0].source,
                     name_token=get_keyword_name_token_from_line(kw[0].lineno),
                     line_no=kw[0].lineno,
@@ -940,7 +1088,7 @@ class ImportsManager:
                     if isinstance(kw[1], UserErrorHandler)
                     else None,
                     arguments=ArgumentSpec.from_robot_argument_spec(kw[1].arguments),
-                    parent=libdoc,
+                    parent=libdoc.digest,
                 )
                 for kw in [(KeywordDocBuilder(resource=True).build_keyword(lw), lw) for lw in lib.handlers]
             ],
@@ -975,7 +1123,7 @@ class ImportsManager:
                         args,
                         str(self.folder.to_path()),
                         base_dir,
-                        self.config.variables if self.config is not None else None,
+                        self.config.robot.variables if self.config.robot is not None else None,
                         variables,
                     ),
                     LOAD_LIBRARY_TIME_OUT,
@@ -1071,7 +1219,7 @@ class ImportsManager:
             name,
             str(self.folder.to_path()),
             base_dir,
-            self.config.variables if self.config is not None else None,
+            self.config.robot.variables if self.config.robot is not None else None,
             variables,
         )
 
@@ -1082,7 +1230,7 @@ class ImportsManager:
             name,
             str(self.folder.to_path()),
             base_dir,
-            self.config.variables if self.config is not None else None,
+            self.config.robot.variables if self.config.robot is not None else None,
             variables,
         )
 
@@ -1094,7 +1242,7 @@ class ImportsManager:
             name,
             str(self.folder.to_path()),
             base_dir,
-            self.config.variables if self.config is not None else None,
+            self.config.robot.variables if self.config.robot is not None else None,
             variables,
         )
 
@@ -1103,6 +1251,6 @@ class ImportsManager:
             name,
             str(self.folder.to_path()),
             base_dir,
-            self.config.variables if self.config is not None else None,
+            self.config.robot.variables if self.config.robot is not None else None,
             variables,
         )
